/**
 *  @filename: SOAPUtils.java
 *  @create date: 2003-09-03
 *  @author: Grants.gov - Dan Dumitru
 *
 *  @version 1.0.0
 *  @revision date:
 *
 */

package gov.grants.webSvcs.utility;

// Sun
import javax.xml.soap.*;
import javax.xml.messaging.*;
import java.util.*;

// Grants.gov
import gov.grants.utility.GrantsGovConstants;

public final class SOAPUtils {

    private SOAPUtils() {}

	 /**********************************
	 *
	 *  :: Retreive SOAP element method::
	 *
     **********************************/

	public static String getSOAPElement( SOAPMessage requestMessage, String element, String function )throws javax.xml.soap.SOAPException, org.xml.sax.SAXException, java.io.IOException {

		String elementValue;
        SOAPPart soapPart;
        SOAPEnvelope soapEnvelope;
        SOAPBody soapBody;
		javax.xml.soap.Name functionName;
		javax.xml.soap.Name elementName;


		soapPart = requestMessage.getSOAPPart();
		soapEnvelope = soapPart.getEnvelope();
        soapBody = soapEnvelope.getBody();
		// Search for Fuction name
		functionName = soapEnvelope.createName( function, GrantsGovConstants.AGIS_PREFIX, GrantsGovConstants.AGIS_URI);

		// Check there's at least one such element.
		java.util.Iterator valueRequests = soapBody.getChildElements(functionName);
		if (! valueRequests.hasNext()) {
		// Schema is not valid
		 }

		// Check there's no more than one of such Function .
		SOAPElement soapGetElement = (SOAPElement)valueRequests.next();
		if (valueRequests.hasNext()) {
		// Schema is not valid
		}

		// Look for the appropriate child element containing the name parameter.
		elementName = soapEnvelope.createName(element);
		java.util.Iterator params = soapGetElement.getChildElements(elementName);
		if (! params.hasNext()) {
		// Schema is not valid
		}

		// Ensure there's only one such element.
		SOAPElement soapElement = (SOAPElement)params.next();
		if (params.hasNext()) {
		// Schema is not valid
		}

		// Attempt to read out the value of element
		elementValue = soapElement.getValue();
		if (null == elementValue) {
		// Schema is not valid
		}

		System.out.println( element + " = " + elementValue );

		return elementValue;
	}

	//Instance for SOAP envelopes that do not include a function element
	public static String getSOAPElement( SOAPMessage requestMessage, String element )throws javax.xml.soap.SOAPException, org.xml.sax.SAXException, java.io.IOException {

		String elementValue;
        SOAPPart soapPart;
        SOAPEnvelope soapEnvelope;
        SOAPBody soapBody;
		javax.xml.soap.Name elementName;


		soapPart = requestMessage.getSOAPPart();
		soapEnvelope = soapPart.getEnvelope();
        soapBody = soapEnvelope.getBody();

		// Look for the appropriate child element containing the name parameter.
		elementName = soapEnvelope.createName(element);
		java.util.Iterator params = soapBody.getChildElements(elementName);
		if (! params.hasNext()) {
		// Schema is not valid
		}

		// Ensure there's only one such element.
		SOAPElement soapElement = (SOAPElement)params.next();
		if (params.hasNext()) {
		// Schema is not valid
		}

		// Attempt to read out the value of element
		elementValue = soapElement.getValue();
		if (null == elementValue) {
		// Schema is not valid
		}

		System.out.println( element + " = " + elementValue );

		return elementValue;
	}


     /**********************************
     *
     *  :: SOAP FAULT Message ::
	 *  :: Generation METHODS ::
     *
     **********************************/

	public static SOAPMessage generateSOAPFaultMsg( String faultString, String localName )
	throws javax.xml.soap.SOAPException {

		// Generate SOAP Fault message
		SOAPMessage responseMsg = generateSOAPFaultMsg( MessageFactory.newInstance(), faultString, localName );
		return responseMsg;
	}

	public static SOAPMessage generateSOAPFaultMsg( MessageFactory messageFactory, String faultString, String localName )
	throws javax.xml.soap.SOAPException {

		// Generate SOAP Fault message
		SOAPMessage responseMsg = messageFactory.createMessage();
		SOAPBody responseBody = responseMsg.getSOAPPart().getEnvelope().getBody();
		SOAPFault fault = responseBody.addFault();
		SOAPFactory soapFactory = SOAPFactory.newInstance();

//		javax.xml.soap.Name faultName = soapFactory.createName( localName, "", SOAPConstants.URI_NS_SOAP_ENVELOPE );
		fault.setFaultCode( localName );
		fault.setFaultString( faultString );

/*
		fault.setFaultActor("http://gizmos.com/order");
		Detail detail = fault.addDetail();

		Name entryName = soapFactory.createName( "order", "PO", "http://gizmos.com/orders/" );
		DetailEntry entry = detail.addDetailEntry(entryName);
		entry.addTextNode( "Quantity element does not have a value" );

		Name entryName2 = soapFactory.createName("confirmation", "PO", "http://gizmos.com/confirm");
		DetailEntry entry2 = detail.addDetailEntry(entryName2);
		entry2.addTextNode("Incomplete address: " + "no zip code");
*/

		responseMsg.saveChanges();
		return responseMsg;
	}

	public static void processSOAPFault(SOAPBody body) throws Exception{

		SOAPFault newFault = body.getFault();

		// Get the qualified name of the fault code
		//Name code = newFault.getFaultCodeAsName();

		String code = newFault.getFaultCode();
		String string = newFault.getFaultString();
		String actor = newFault.getFaultActor();
		String e = "SOAP fault contains: \n";
		e = e + "  Fault code = " + code + ";\n";
		e = e + "  Fault string = " + string + ";\n";
		if ( actor != null ) {
			e = e + "  Fault actor = " + actor + ";\n";
		}
		Detail newDetail = newFault.getDetail();
		if ( newDetail != null) {
			Iterator entries =
				newDetail.getDetailEntries();
			while ( entries.hasNext() ) {
				DetailEntry newEntry =
					(DetailEntry)entries.next();
				String value = newEntry.getValue();
				e = e + "  Detail entry = " + value + ";\n";
			}
		}

		throw new Exception( e );
	}

}// SOAPUtils