/*
 *  UploadApplicationServlet.java
 *
 *  @author :: Dan Dumitru ::
 *  Created on 2003-07-14
 *	
 *	This servlet performs the following high-level tasks:
 *  - uploads the XML file;
 *  - unmarshals the XML file to JAXB generated java objects
 *  - saves the java objects to the database via a Mediator class
 * 
 */
 package gov.grants.xmlToDB.servlet;

// sun
import java.io.*;
import java.util.*;
import java.util.*;
import java.net.*;
import java.sql.*;
import java.text.*;

import javax.servlet.http.*;
import javax.servlet.*;

// reference implementation
import gov.grants.utility.GrantApplicationProcessor;
import gov.grants.xmlToDB.utility.MultipartRequest;
import gov.grants.xmlToDB.utility.DaoMediator;

public class UploadApplicationServlet extends HttpServlet
{

	private GrantApplicationProcessor processor;
	private RequestDispatcher rd;
	
	public void doGet( HttpServletRequest request, HttpServletResponse response ) throws IOException,ServletException {
		doPost( request, response );
	}
	
	public void doPost(HttpServletRequest request, HttpServletResponse response ) throws IOException,ServletException	{
		String filename = "";
		String grantsGovTrackingNumber;
		
		try {
			
			// Upload the XML file
			MultipartRequest multiRequest = new MultipartRequest( request, "." , 10*1024*1024 );
			Enumeration files = multiRequest.getFileNames();
			
			if (files.hasMoreElements()) {
				
				String name = (String)files.nextElement();
				filename = multiRequest.getFilesystemName( name );
				String type = multiRequest.getContentType( name );
				File file = multiRequest.getFile(name);
				
				// Debug  
				System.out.println( "name	= " + name );
				System.out.println( "filename = " + filename);
				System.out.println( "type =" + type );
				
				if( file != null ) {
					System.out.println( "File length = " + file.length() );
					System.out.println();
					// Unmarshal XML file to JAXB generated java objects
					processor = new GrantApplicationProcessor( GrantApplicationProcessor.OTHER_SOURCE, GrantApplicationProcessor.GRANT_APPLICATION_CONTEXT_PATH );
					// processor.unmarshal( filename );
					processor.unmarshal( file );
					System.out.println( "XML File has been parsed/unsmarshalled; about to save to database" );
					DaoMediator.createApplication( processor.getGrantApplication() );
				}
			} // end while()

		} catch( Exception e ) { 
			
			System.out.println( "ERROR found processing XML file: " + e.getMessage() );
			
			request.setAttribute( "ErrorMsg", e.getMessage() );
			
			System.out.println( "About to forward to /xmlToDB/jsp/include/xmlToDBErrorPage.jsp" );
			rd = request.getRequestDispatcher( "/xmlToDB/jsp/include/xmlToDBErrorPage.jsp" );
			rd.forward(request, response);
			return;
			
		} finally {
			// delete the uploaded application xml file
			if (( new File("./" + filename)).delete() ) {
				System.out.println( "Delete complete" );
			} else { 
				System.out.println( "Delete Failed" );
			}
			
			// Cleanup - Release Processor
			processor.resetGrantApplication();
			processor = null;
		}

		System.out.println( "About to forward to /xmlToDB/jsp/Import.jsp" );
		rd = request.getRequestDispatcher( "/xmlToDB/jsp/Import.jsp" );
		rd.forward(request, response);

	}
}// UploadApplicationServlet

